*DECK COMDEP
      SUBROUTINE COMDEP(IPRINT,IPEDIT,IPWORK,ITRES,NISOP,NOMEVO)
*
*-----------------------------------------------------------------------
*
*Purpose:
* creation of a lumped depletion chain in the multicompo.
*
*Copyright:
* Copyright (C) 2015 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version.
*
*Author(s): A. Hebert
*
*Parameters: input/output
* IPRINT  print parameter
* IPEDIT  pointer to the edition object (L_EDIT signature).
* IPWORK  pointer to the LCM object where the lumped depletion chain is
*         written.
* ITRES   creation index for the macroscopic residual (=0: not created;
*         =1: not a FP precursor; =2: is a FP precursor).
* NISOP   number of user-requested particularized isotopes. Equal to
*         zero if all EDI: isotopes are particularized.
* NOMEVO  library names of user-requested particularized isotopes.
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER IPRINT,IPEDIT,IPWORK,ITRES,NISOP
      CHARACTER NOMEVO(NISOP)*12
*----
*  LOCAL VARIABLES
*----
      PARAMETER (NSTATE=40,MAXBCH=500)
      INTEGER ISTATE(NSTATE),IHICH(3,MAXBCH),IBASE(1)
      LOGICAL LISO
      CHARACTER TEXT12*12
      COMMON BASE(1)
      EQUIVALENCE(IBASE(1),BASE(1))
*----
*  RECOVER DEPLETION INFORMATION FROM EDITION OBJECT
*----
      CALL LCMSIX(IPEDIT,'DEPL-CHAIN',1)
      IF(NISOP.GT.0) THEN
         CALL LCMGET(IPEDIT,'STATE-VECTOR',ISTATE)
         NBISO=ISTATE(1)
         IF(ITRES.EQ.2) NBISO=NBISO+1
         NBFISS=ISTATE(2)
         NBDPF=ISTATE(3)
         NSUPS=ISTATE(7)
         NREAC=ISTATE(8)
         NFATH=ISTATE(9)
         MAXFP=NBDPF+30 ! reserve 30 location for lumped fp daughters
         CALL SETARA(IBASE,3*NBISO,IIHISO)
         CALL SETARA(IBASE,NBISO,IMYLIS)
         CALL SETARA(IBASE,2*NREAC,IHREAC)
         CALL SETARA(IBASE,NREAC*NBISO,IIDREA)
         CALL SETARA(BASE,NREAC*NBISO,IDENER)
         CALL SETARA(BASE,NBISO,IDDECA)
         CALL SETARA(IBASE,NFATH*NBISO,IIPREA)
         CALL SETARA(BASE,NFATH*NBISO,IPRATE)
         CALL SETARA(BASE,NBFISS*MAXFP,IYIELD)
         CALL LCMGET(IPEDIT,'ISOTOPESDEPL',IBASE(IIHISO))
         CALL LCMGET(IPEDIT,'CHARGEWEIGHT',IBASE(IMYLIS))
         CALL LCMGET(IPEDIT,'DEPLETE-IDEN',IBASE(IHREAC))
         CALL LCMGET(IPEDIT,'DEPLETE-REAC',IBASE(IIDREA))
         CALL LCMGET(IPEDIT,'DEPLETE-ENER',BASE(IDENER))
         CALL LCMGET(IPEDIT,'DEPLETE-DECA',BASE(IDDECA))
         CALL LCMGET(IPEDIT,'PRODUCE-REAC',IBASE(IIPREA))
         CALL LCMGET(IPEDIT,'PRODUCE-RATE',BASE(IPRATE))
         IF(NBFISS*NBDPF.GT.0) THEN
            CALL LCMGET(IPEDIT,'FISSIONYIELD',BASE(IYIELD))
         ENDIF
*----
*  DESCRIBE FISSILE ISOTOPE *MAC*RES
*----
         IF(ITRES.EQ.2) THEN
           IF(IPRINT.GT.1) THEN
              WRITE(6,'(/42H COMDEP: ADD *MAC*RES RESIDUAL ISOTOPE TO ,
     1        17HDEPLETION CHAINS.)')
            ENDIF
            TEXT12='*MAC*RES'
            READ(TEXT12,'(3A4)') (IBASE(IIHISO+(NBISO-1)*3+I0-1),I0=1,3)
            IBASE(IMYLIS+NBISO-1)=0
            DO I=1,NREAC
               IBASE(IIDREA+(NBISO-1)*NREAC+I-1)=0
               BASE(IDENER+(NBISO-1)*NREAC+I-1)=0.0
            ENDDO
            IBASE(IIDREA+(NBISO-1)*NREAC)=4
            BASE(IDDECA+NBISO-1)=0.0
            DO I=1,NFATH
               IBASE(IIPREA+(NBISO-1)*NREAC+I-1)=0
               BASE(IPRATE+(NBISO-1)*NREAC+I-1)=0.0
            ENDDO
         ENDIF
*----
*  CREATE LUMPED DEPLETION CHAIN
*----
         CALL LCMSIX(IPWORK,'DEPL-CHAIN',1)
         LISO=.FALSE.
         NBCH=0
         DO 20 ISO=1,NBISO
         WRITE(TEXT12,'(3A4)') (IBASE(IIHISO+(ISO-1)*3+I0-1),I0=1,3)
         DO JSO=1,NISOP
           IF((TEXT12.EQ.NOMEVO(JSO)).AND.(TEXT12.NE.'*MAC*RES')) THEN
             NBCH=NBCH+1
             IF(NBCH.GT.MAXBCH) CALL XABORT('COMDEP: MAXBCH OVERFLOW.')
             READ(TEXT12,'(3A4)') (IHICH(I0,NBCH),I0=1,3)
             GO TO 20
           ENDIF
         ENDDO
         IF((TEXT12.EQ.'*MAC*RES').AND.(ITRES.EQ.2)) THEN
           NBCH=NBCH+1
           IF(NBCH.GT.MAXBCH) CALL XABORT('COMDEP: MAXBCH OVERFLOW.')
           READ(TEXT12,'(3A4)') (IHICH(I0,NBCH),I0=1,3)
         ENDIF
   20    CONTINUE
         CALL SETARA(IBASE,MAXFAT*NBCH,IJPREA)
         CALL SETARA(IBASE,MAXFAT*NBCH,IRRATE)
         CALL SETARA(IBASE,NREAC*NBCH,IJDREA)
         CALL SETARA(IBASE,NREAC*NBCH,IEENER)
         CALL SETARA(IBASE,NBFISS*NBCH,IEYIEL)
         CALL SETARA(IBASE,NBCH*3,IIPOS)
         CALL SETARA(IBASE,3*NBCH,IHHHH)
         CALL SETARA(IBASE,NBCH,IDDDD)
         CALL EDILUM(IPRINT,IPWORK,MAXFP,NBISO,NBFISS,NBDPF,NSUPS,
     1   NREAC,NFATH,NBCH,IHICH,IBASE(IIHISO),IBASE(IMYLIS),
     2   IBASE(IHREAC),IBASE(IIDREA),IBASE(IDENER),IBASE(IDDECA),
     3   IBASE(IIPREA),IBASE(IPRATE),IBASE(IYIELD),IBASE(IJPREA),
     4   IBASE(IRRATE),IBASE(IJDREA),IBASE(IEENER),IBASE(IEYIEL),
     5   IBASE(IIPOS),IBASE(IHHHH),IBASE(IDDDD),LISO,NBFISS,NBCH)
         CALL RLSARA(IBASE(IDDDD))
         CALL RLSARA(IBASE(IHHHH))
         CALL RLSARA(IBASE(IIPOS))
         CALL RLSARA(IBASE(IEYIEL))
         CALL RLSARA(IBASE(IEENER))
         CALL RLSARA(IBASE(IJDREA))
         CALL RLSARA(IBASE(IRRATE))
         CALL RLSARA(IBASE(IJPREA))
*
         CALL RLSARA(BASE(IYIELD))
         CALL RLSARA(BASE(IPRATE))
         CALL RLSARA(IBASE(IIPREA))
         CALL RLSARA(BASE(IDDECA))
         CALL RLSARA(BASE(IDENER))
         CALL RLSARA(IBASE(IIDREA))
         CALL RLSARA(IBASE(IHREAC))
         CALL RLSARA(IBASE(IMYLIS))
         CALL RLSARA(IBASE(IIHISO))
      ELSE
*----
*  RECOVER THE DEPLETION CHAIN WITHOUT LUMPING
*----
         CALL LCMSIX(IPWORK,'DEPL-CHAIN',1)
         CALL LCMEQU(IPEDIT,IPWORK)
      ENDIF
      CALL LCMSIX(IPWORK,' ',2)
      CALL LCMSIX(IPEDIT,' ',2)
      RETURN
      END
