*DECK COMRES
      SUBROUTINE COMRES(IMPX,IPISO,IPEDIT,NISOTS,NG,NL,NED,NBESP,NDEL,
     1 NISO2,ISW,ITRES,PYRES,GAS,IHNISO,SDEN,WSCAT,PNFIRA,PNFTOT,ITYPRO,
     2 WORK,WORK2)
*
*-----------------------------------------------------------------------
*
*Purpose:
* compute a non-depleting macroscopic residual isotope.
*
*Copyright:
* Copyright (C) 2015 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input
* IMPX    print parameter.
* IPISO   pointer to the isotope directory containing the residual
*         isotope.
* IPEDIT  pointer to the edition object (L_EDIT signature).
* NISOTS  number of isotopes in the microlib pointed by IPEDIT.
* NG      number of energy groups.
* NL      number of Legendre orders.
* NED     number of extra edit reactions.
* NBESP   number of energy-dependent fission spectra.
* NDEL    number of delayed-neutron precursor groups.
* NISO2   number of particularized isotopes.
* ISW     selection flag for isotopes in IPEDIT (>0: particularized;
*         <0: included in the macroscopic residual).
*
*Parameters: output
* ITRES   type of the residual isotope (=1: not fissile; =2: fissile).
* PYRES   fission yields of the residual isotope (if ITRES=2).
*
*Parameters: scratch
* GAS
* IHNISO
* SDEN
* WSCAT
* PNFIRA
* PNFTOT
* ITYPRO
* WORK
* WORK2
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER IMPX,IPISO,IPEDIT,NISOTS,NG,NL,NED,NBESP,NDEL,NISO2,
     1 ISW(NISOTS),ITRES,IHNISO(3,NISOTS),ITYPRO(NL)
      REAL PYRES(NISO2),GAS(NG,12+NL+NED+2*NDEL+NBESP),SDEN(NISOTS),
     1 WSCAT(NG,NG,NL),PNFIRA(NG,0:NDEL,2),PNFTOT(NISOTS),WORK(NG+1),
     2 WORK2(NG,NG,NL)
*----
*  LOCAL VARIABLES
*----
      PARAMETER (NSTATE=40,MAXED=100,MAXHMA=100)
      INTEGER ISTATE(NSTATE),IBASE(1)
      CHARACTER CM*2,TEXT12*12,HVECT(MAXED)*8,HMAKE(MAXHMA)*12
      COMMON RBASE(1)
      EQUIVALENCE (RBASE(1),IBASE(1))
*
      MAXH=12+NL+NED+2*NDEL+NBESP
      IF(NED.GT.MAXED) CALL XABORT('COMRES: MAXED OVERFLOW.')
      IF(MAXH+NL.GT.MAXHMA) CALL XABORT('COMRES: MAXHMA OVERFLOW.')
      CALL LCMGET(IPEDIT,'ISOTOPESUSED',IHNISO)
      CALL LCMGET(IPEDIT,'ISOTOPESDENS',SDEN)
      CALL LCMGTC(IPEDIT,'ADDXSNAME-P0',8,NED,HVECT)
      HMAKE(:MAXH+NL)=' '
      CALL XDRSET(GAS,MAXH*NG,0.0)
      CALL XDRSET(WSCAT,NL*NG*NG,0.0)
      CALL XDRSET(PNFIRA(1,0,2),NG*(NDEL+1),0.0)
      CALL XDRSET(PNFTOT,NISOTS,0.0)
      DENTOT=0.0
      DAWR=0.0
      DECISO=0.0
      ITRES=1
*----
*  SUMMATION OVER NON-PARTICULARIZED ISOTOPES
*----
      DO 170 ISO=1,NISOTS
      IF(ISW(ISO).LT.0) THEN
         ISO3=-ISW(ISO)
         IF(ISO3.GT.NISOTS) CALL XABORT('COMRES: NISOTS OVERFLOW(1).')
         DDEN=SDEN(ISO)
         WRITE(TEXT12,'(3A4)') (IHNISO(I0,ISO),I0=1,3)
         CALL LCMSIX(IPEDIT,TEXT12,1)
         CALL LCMLEN(IPEDIT,'AWR',LENGTH,ITYLCM)
         IF(LENGTH.EQ.1) THEN
            CALL LCMGET(IPEDIT,'AWR',FLOTT)
            DAWR=DAWR+DDEN*FLOTT
         ENDIF
         DENTOT=DENTOT+DDEN
         CALL LCMLEN(IPEDIT,'DECAY',LENGTH,ITYLCM)
         IF(LENGTH.EQ.1) THEN
            CALL LCMGET(IPEDIT,'DECAY',FLOTT)
            DECISO=DECISO+FLOTT*DDEN
         ENDIF
*
*        COMPUTE FISSION RATES.
         CALL LCMLEN(IPEDIT,'NFTOT',LENGTH,ITYLCM)
         IF(LENGTH.EQ.NG) THEN
            CALL LCMGET(IPEDIT,'NWT0',GAS(1,1))
            CALL LCMGET(IPEDIT,'NFTOT',WORK)
            DO 40 IGR=1,NG
            DEL=WORK(IGR)*GAS(IGR,1)*DDEN
   40       PNFTOT(ISO3)=PNFTOT(ISO3)+DEL
         ENDIF
*
*        SET ARRAY HMAKE.
         CALL LCMLEN(IPEDIT,'NWT0',ILONG,ITYLCM)
         IF(ILONG.EQ.NG) HMAKE(1)='NWT0'
         CALL LCMLEN(IPEDIT,'NWT1',ILONG,ITYLCM)
         IF(ILONG.EQ.NG) HMAKE(2)='NWT1'
         CALL LCMLEN(IPEDIT,'NWAT0',ILONG,ITYLCM)
         IF(ILONG.EQ.NG) HMAKE(3)='NWAT0'
         CALL LCMLEN(IPEDIT,'NWAT1',ILONG,ITYLCM)
         IF(ILONG.EQ.NG) HMAKE(4)='NWAT1'
         IOF=4
         DO 50 IL=0,NL-1
         IOF=IOF+1
         WRITE (CM,'(I2.2)') IL
         CALL LCMLEN(IPEDIT,'SIGS'//CM,ILONG,ITYLCM)
         IF(ILONG.EQ.NG) HMAKE(IOF)='SIGS'//CM
   50    CONTINUE
         CALL LCMLEN(IPEDIT,'NTOT0',ILONG,ITYLCM)
         IF(ILONG.EQ.NG) HMAKE(IOF+1)='NTOT0'
         CALL LCMLEN(IPEDIT,'NTOT1',ILONG,ITYLCM)
         IF(ILONG.EQ.NG) HMAKE(IOF+2)='NTOT1'
         CALL LCMLEN(IPEDIT,'NUSIGF',ILONG,ITYLCM)
         IF(ILONG.EQ.NG) HMAKE(IOF+3)='NUSIGF'
         IOF=IOF+3
         DO 60 IED=1,NED
         IOF=IOF+1
         CALL LCMLEN(IPEDIT,HVECT(IED),ILONG,ITYLCM)
         IF(ILONG.EQ.NG) HMAKE(IOF)=HVECT(IED)
   60    CONTINUE
         CALL LCMLEN(IPEDIT,'H-FACTOR',ILONG,ITYLCM)
         IF(ILONG.EQ.NG) HMAKE(IOF+1)='H-FACTOR'
         CALL LCMLEN(IPEDIT,'OVERV',ILONG,ITYLCM)
         IF(ILONG.EQ.NG) HMAKE(IOF+2)='OVERV'
         CALL LCMLEN(IPEDIT,'TRANC',ILONG,ITYLCM)
         IF(ILONG.EQ.NG) HMAKE(IOF+3)='TRANC'
         IOF=IOF+3
         DO 70 IDEL=1,NDEL
         IOF=IOF+1
         WRITE(TEXT12,'(6HNUSIGF,I2.2)') IDEL
         CALL LCMLEN(IPEDIT,TEXT12,ILONG,ITYLCM)
         IF(ILONG.EQ.NG) HMAKE(IOF)=TEXT12
   70    CONTINUE
         IOF=IOF+1
         CALL LCMLEN(IPEDIT,'CHI',ILONG,ITYLCM)
         IF(ILONG.EQ.NG) HMAKE(IOF)='CHI'
         DO 80 IDEL=1,NDEL
         IOF=IOF+1
         WRITE(TEXT12,'(3HCHI,I2.2)') IDEL
         CALL LCMLEN(IPEDIT,TEXT12,ILONG,ITYLCM)
         IF(ILONG.EQ.NG) HMAKE(IOF)=TEXT12
   80    CONTINUE
         DO 85 ISP=1,NBESP
         IOF=IOF+1
         WRITE(TEXT12,'(5HCHI--,I2.2)') ISP
         CALL LCMLEN(IPEDIT,TEXT12,ILONG,ITYLCM)
         IF(ILONG.EQ.NG) HMAKE(IOF)=TEXT12
   85    ENDDO
         CALL LCMLEN(IPEDIT,'STRD',ILONG,ITYLCM)
         IF(ILONG.EQ.NG) HMAKE(IOF+1)='STRD'
*
         DO 150 J=1,MAXH
         IF(HMAKE(J).NE.' ') THEN
           CALL LCMLEN(IPEDIT,HMAKE(J),ILONG,ITYLCM)
           IF(ILONG.GT.0) THEN
             CALL LCMGET(IPEDIT,HMAKE(J),WORK)
             IF(HMAKE(J).EQ.'NUSIGF') THEN
               DO 90 IGR=1,NG
               DEL=WORK(IGR)*GAS(IGR,1)*DDEN
               PNFIRA(IGR,0,1)=DEL
               PNFIRA(IGR,0,2)=PNFIRA(IGR,0,2)+DEL
   90          GAS(IGR,J)=GAS(IGR,J)+WORK(IGR)*DDEN
             ELSE IF(HMAKE(J)(:3).EQ.'NUS') THEN
               IDEL=J-(10+NL+NED)
               DO 100 IGR=1,NG
               DEL=WORK(IGR)*GAS(IGR,1)*DDEN
               PNFIRA(IGR,IDEL,1)=DEL
               PNFIRA(IGR,IDEL,2)=PNFIRA(IGR,IDEL,2)+DEL
  100          GAS(IGR,J)=GAS(IGR,J)+WORK(IGR)*DDEN
             ELSE IF(HMAKE(J)(:3).EQ.'NWT') THEN
               DO 110 IGR=1,NG
  110          GAS(IGR,J)=WORK(IGR)
             ELSE IF((HMAKE(J).EQ.'CHI').OR.
     1               (HMAKE(J)(:5).EQ.'CHI--')) THEN
               DO 120 IGR=1,NG
  120          GAS(IGR,J)=GAS(IGR,J)+WORK(IGR)*PNFIRA(IGR,0,1)
             ELSE IF(HMAKE(J)(:3).EQ.'CHI') THEN
               IDEL=J-(11+NL+NED+NDEL)
               DO 130 IGR=1,NG
  130          GAS(IGR,J)=GAS(IGR,J)+WORK(IGR)*PNFIRA(IGR,IDEL,1)
             ELSE
               DO 140 IGR=1,NG
  140            GAS(IGR,J)=GAS(IGR,J)+WORK(IGR)*DDEN
             ENDIF
           ENDIF
         ENDIF
  150    CONTINUE
         CALL LCMLEN(IPEDIT,'SCAT-SAVED',ILONG,ITYLCM)
         IF(ILONG.GT.0) THEN
           CALL SETARA(RBASE,NG*NL,ISIGS)
           CALL SETARA(RBASE,NG*NG,IXSSCM)
           CALL SETARA(RBASE,NG,INJJ)
           CALL SETARA(RBASE,NG,IIJJ)
           CALL XDRLGS(IPEDIT,-1,IMPX,0,NL-1,1,NG,RBASE(ISIGS),WORK2,
     1     ITYPRO,RBASE(IXSSCM),RBASE(INJJ),RBASE(IIJJ))
           CALL RLSARA(RBASE(IIJJ))
           CALL RLSARA(RBASE(INJJ))
           CALL RLSARA(RBASE(IXSSCM))
           CALL RLSARA(RBASE(ISIGS))
           DO 160 IL=1,NL
           WRITE (CM,'(I2.2)') IL-1
           IF(ITYPRO(IL).NE.0) HMAKE(MAXH+IL)='SCAT'//CM
           DO 160 JGR=1,NG
           DO 160 IGR=1,NG
  160      WSCAT(IGR,JGR,IL)=WSCAT(IGR,JGR,IL)+WORK2(IGR,JGR,IL)*DDEN
         ENDIF
         CALL LCMSIX(IPEDIT,' ',2)
      ENDIF
  170 CONTINUE
*----
*  SUMMATION OF FISSION YIELDS OVER PARTICULARIZED ISOTOPES
*----
      CALL SETARA(RBASE,NISO2,IPYDEN)
      CALL XDRSET(PYRES,NISO2,0.0)
      CALL XDRSET(RBASE(IPYDEN),NISO2,0.0)
      DO 190 ISO=1,NISOTS
      IF(ISW(ISO).GT.0) THEN
        ISO2=ISW(ISO)
        IF(ISO2.GT.NISOTS) CALL XABORT('COMRES: NISOTS OVERFLOW(2).')
        WRITE(TEXT12,'(3A4)') (IHNISO(I0,ISO),I0=1,3)
        CALL LCMSIX(IPEDIT,TEXT12,1)
        CALL LCMLEN(IPEDIT,'PIFI',NDFI,ITYLCM)
        IF(NDFI.GT.0) THEN
          CALL SETARA(RBASE,NDFI,IIPIFI)
          CALL SETARA(RBASE,NDFI,IPYIEL)
          CALL LCMGET(IPEDIT,'PIFI',IBASE(IIPIFI))
          CALL LCMGET(IPEDIT,'PYIELD',RBASE(IPYIEL))
          DO 180 I=1,NDFI
            IFI=IBASE(IIPIFI+I-1)
            IF(IFI.GT.NISOTS) CALL XABORT('COMRES: NISOTS OVERFLOW(3).')
            IF(ISW(IFI).LT.0) THEN
*            fissile isotope included in the macroscopic residual
             PYRES(ISO2)=PYRES(ISO2)+RBASE(IPYIEL+I-1)*PNFTOT(-ISW(IFI))
             RBASE(IPYDEN+ISO2-1)=RBASE(IPYDEN+ISO2-1)+PNFTOT(-ISW(IFI))
            ENDIF
  180     CONTINUE
          CALL RLSARA(RBASE(IPYIEL))
          CALL RLSARA(IBASE(IIPIFI))
        ENDIF
        CALL LCMSIX(IPEDIT,' ',2)
      ENDIF
  190 CONTINUE
      DO 200 ISO2=1,NISO2
      IF(RBASE(IPYDEN+ISO2-1).NE.0.0) THEN
        PYRES(ISO2)=PYRES(ISO2)/RBASE(IPYDEN+ISO2-1)
      ENDIF
  200 CONTINUE
      CALL RLSARA(RBASE(IPYDEN))
*----
*  SAVE THE RESIDUAL ISOTOPE IN IPISO ISOTOPE OBJECT
*----
      IF(DENTOT.GT.0.0) THEN
         IF(IMPX.GT.0) WRITE (6,600)
         DAWR=DAWR/DENTOT
         CALL LCMPUT(IPISO,'AWR',1,2,DAWR)
         DECISO=DECISO/DENTOT
         IF(DECISO.GT.0.0) CALL LCMPUT(IPISO,'DECAY',1,2,DECISO)
         DO 240 J=1,MAXH
         IF(HMAKE(J).EQ.'NUSIGF') ITRES=2
         IF(HMAKE(J).EQ.'OVERV') THEN
            DO 210 IGR=1,NG
  210       GAS(IGR,J)=GAS(IGR,J)/DENTOT
         ELSE IF((HMAKE(J).EQ.'CHI').OR.(HMAKE(J)(:5).EQ.'CHI--')) THEN
            DO 220 IGR=1,NG
            IF(GAS(IGR,J).NE.0.0) THEN
               GAS(IGR,J)=GAS(IGR,J)/PNFIRA(IGR,0,2)
            ENDIF
  220       CONTINUE
         ELSE IF(HMAKE(J)(:3).EQ.'CHI') THEN
            IDEL=J-(11+NL+NED+NDEL)
            DO 230 IGR=1,NG
            IF(GAS(IGR,J).NE.0.0) THEN
               GAS(IGR,J)=GAS(IGR,J)/PNFIRA(IGR,IDEL,2)
            ENDIF
  230       CONTINUE
         ENDIF
         IF((HMAKE(J).NE.' ').AND.(HMAKE(J)(:4).NE.'SIGS')) THEN
            CALL LCMPUT(IPISO,HMAKE(J),NG,2,GAS(1,J))
         ENDIF
  240    CONTINUE
         IF(ITYPRO(1).NE.0) THEN
            DO 250 IL=1,NL
            ITYPRO(IL)=0
            IF(HMAKE(MAXH+IL).NE.' ') ITYPRO(IL)=1
  250       CONTINUE
            CALL SETARA(RBASE,NG*NG,IXSSCM)
            CALL SETARA(RBASE,NG,INJJ)
            CALL SETARA(RBASE,NG,IIJJ)
            CALL XDRLGS(IPISO,1,IMPX,0,NL-1,1,NG,GAS(1,5),WSCAT,ITYPRO,
     >      RBASE(IXSSCM),RBASE(INJJ),RBASE(IIJJ))
            CALL RLSARA(RBASE(IIJJ))
            CALL RLSARA(RBASE(INJJ))
            CALL RLSARA(RBASE(IXSSCM))
         ENDIF
*
         IF(IMPX.GT.3) THEN
            WRITE(6,'(/17H NUMBER DENSITY =,1P,E12.4)') 1.0
            WRITE(6,'(23H WEIGHTED ATOMIC MASS =,1P,E13.5)') DAWR
            DO 260 J=1,MAXH
            IF(HMAKE(J).NE.' ') THEN
               WRITE (6,610) HMAKE(J),(GAS(I,J),I=1,NG)
            ENDIF
  260       CONTINUE
            WRITE (6,610) 'SIGA    ',(GAS(I,5+NL)-GAS(I,5),I=1,NG)
            WRITE (6,610) 'SIGW00  ',(WSCAT(I,I,1),I=1,NG)
            IF(NL.GT.1) THEN
               WRITE (6,610) 'SIGW01  ',(WSCAT(I,I,2),I=1,NG)
            ENDIF
         ENDIF
      ENDIF
      RETURN
*
  600 FORMAT (//49H COMRES: CROSS SECTION OF MACRO RESIDUAL ISOTOPE ,
     1 8H*MAC*RES)
  610 FORMAT (/11H REACTION ',A12,2H':/(1X,1P,10E12.4))
      END
