*DECK DUO006
      SUBROUTINE DUO006(IPLIB,IPRINT,NISOT,NGRP,HREAC,IDIV,RHS,FLUX,
     > AFLUX,IHUSED,DENS,VOL,VECTOR,SCAT,CHI,RATE)
*
*-----------------------------------------------------------------------
*
*Purpose:
* processing one of the two microlibs and return the RHS matrix for
* the single reaction HREAC.
*
*Copyright:
* Copyright (C) 2013 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input
* IPLIB   microlib
* IPRINT  print parameter
* NISOT   number of isotopes
* NGRP    number of energy groups
* HREAC   character*8 reaction name of the reaction to process
* IDIV    type of divergence term processing (=0: no processing;
*         =1: direct processing; =2: adjoint processing;
*         =3: direct-adjoint processing)
*
*Parameters: output
* RHS     macroscopic cross-section matrix
* FLUX    integrated direct flux
* AFLUX   integrated adjoint flux
*
*Parameters: scratch
* IHUSED
* DENS
* VOL
* VECTOR
* SCAT
* CHI
* RATE
*
*-----------------------------------------------------------------------
*
*----
*  SUBROUTINE ARGUMENTS
*----
      INTEGER IPLIB
      INTEGER IPRINT,NISOT,NGRP,IDIV,IHUSED(3,NISOT)
      CHARACTER HREAC*8
      REAL RHS(NGRP,NGRP,NISOT),FLUX(NGRP,NISOT),AFLUX(NGRP,NISOT),
     > DENS(NISOT),VOL(NISOT),VECTOR(NGRP),SCAT(NGRP,NGRP),CHI(NGRP),
     > RATE(NGRP,NGRP)
*----
*  LOCAL VARIABLES
*----
      CHARACTER TEXT12*12,HSMG*131
      DOUBLE PRECISION SUM
      COMMON BASE(1)
*----
*  COMPUTE THE RHS AND LHS MATRICES
*----
      CALL LCMGET(IPLIB,'ISOTOPESUSED',IHUSED)
      CALL LCMGET(IPLIB,'ISOTOPESDENS',DENS)
      CALL LCMGET(IPLIB,'ISOTOPESVOL',VOL)
      CALL LCMGET(IPLIB,'K-EFFECTIVE',ZKEFF)
      IF(IPRINT.GT.4) WRITE(6,'(35H DUO006: EFFECTIVE MULTIPLICATION F,
     > 6HACTOR=,1P,E12.5)') ZKEFF
      CALL XDRSET(RHS,NGRP*NGRP*NISOT,0.0)
      DO ISOT=1,NISOT
        WRITE(TEXT12,'(3A4)') (IHUSED(I0,ISOT),I0=1,3)
        IF(IPRINT.GT.4) WRITE(6,'(29H DUO006: PROCESSING ISOTOPE '',
     >  A12,2H''.)') TEXT12
        CALL LCMSIX(IPLIB,TEXT12,1)
        CALL LCMGET(IPLIB,'NWT0',FLUX(1,ISOT))
        CALL LCMLEN(IPLIB,'NWAT0',ILON,ITYLCM)
        IF(ILON.NE.0) THEN
          CALL LCMGET(IPLIB,'NWAT0',AFLUX(1,ISOT))
        ELSE
          CALL XDRSET(AFLUX(1,ISOT),NGRP,1.0)
        ENDIF
        DO IGR=1,NGRP
          FLUX(IGR,ISOT)=FLUX(IGR,ISOT)*VOL(ISOT)
          AFLUX(IGR,ISOT)=AFLUX(IGR,ISOT)*VOL(ISOT)
        ENDDO
        CALL LCMLEN(IPLIB,HREAC,ILONG,ITYLCM)
        IF((ILONG.EQ.0).AND.(HREAC.NE.'LEAK')) THEN
          CALL LCMSIX(IPLIB,' ',2)
          CYCLE
        ENDIF
        IF(HREAC.EQ.'SCAT00') THEN
          CALL SETARA(BASE,NGRP,ISIGS)
          CALL SETARA(BASE,NGRP*NGRP,IXSSCM)
          CALL SETARA(BASE,NGRP,INJJ)
          CALL SETARA(BASE,NGRP,IIJJ)
          CALL XDRLGS(IPLIB,-1,IPRINT,0,0,1,NGRP,BASE(ISIGS),SCAT,
     >    ITYPRO,BASE(IXSSCM),BASE(INJJ),BASE(IIJJ))
          CALL RLSARA(BASE(IIJJ))
          CALL RLSARA(BASE(INJJ))
          CALL RLSARA(BASE(IXSSCM))
          CALL RLSARA(BASE(ISIGS))
          DO IGR=1,NGRP
            DO JGR=1,NGRP
              RHS(JGR,IGR,ISOT)=RHS(JGR,IGR,ISOT)+DENS(ISOT)*
     >        SCAT(JGR,IGR)
            ENDDO
          ENDDO
        ELSE IF((HREAC.EQ.'NUSIGF').OR.(HREAC.EQ.'CHI')) THEN
          CALL LCMGET(IPLIB,'NUSIGF',VECTOR)
          CALL LCMGET(IPLIB,'CHI',CHI)
          DO IGR=1,NGRP
            DO JGR=1,NGRP
              RHS(JGR,IGR,ISOT)=RHS(JGR,IGR,ISOT)+DENS(ISOT)*CHI(JGR)*
     >        VECTOR(IGR)
            ENDDO
          ENDDO
        ELSE IF(HREAC(:3).EQ.'NWT') THEN
          WRITE(HSMG,'(8HDUO006: ,A8,25H IS A FORBIDDEN REACTION.)')
     >    HREAC
          CALL XABORT(HSMG)
        ELSE IF(HREAC.EQ.'LEAK') THEN
          CALL XDRSET(RATE,NGRP*NGRP,0.0)
          CALL LCMLEN(IPLIB,'NUSIGF',ILON,ITYLCM)
          IF(ILON.GT.0) THEN
            CALL LCMGET(IPLIB,'NUSIGF',VECTOR)
            CALL LCMGET(IPLIB,'CHI',CHI)
            DO IGR=1,NGRP
              DO JGR=1,NGRP
                RATE(JGR,IGR)=RATE(JGR,IGR)-DENS(ISOT)*CHI(JGR)*
     >          VECTOR(IGR)/ZKEFF
              ENDDO
            ENDDO
          ENDIF
          CALL SETARA(BASE,NGRP,ISIGS)
          CALL SETARA(BASE,NGRP*NGRP,IXSSCM)
          CALL SETARA(BASE,NGRP,INJJ)
          CALL SETARA(BASE,NGRP,IIJJ)
          CALL XDRLGS(IPLIB,-1,IPRINT,0,0,1,NGRP,BASE(ISIGS),SCAT,
     >    ITYPRO,BASE(IXSSCM),BASE(INJJ),BASE(IIJJ))
          CALL RLSARA(BASE(IIJJ))
          CALL RLSARA(BASE(INJJ))
          CALL RLSARA(BASE(IXSSCM))
          CALL RLSARA(BASE(ISIGS))
          CALL LCMGET(IPLIB,'NTOT0',VECTOR)
          DO IGR=1,NGRP
            DO JGR=1,NGRP
              RATE(JGR,IGR)=RATE(JGR,IGR)-DENS(ISOT)*SCAT(JGR,IGR)
            ENDDO
            RATE(IGR,IGR)=RATE(IGR,IGR)+DENS(ISOT)*VECTOR(IGR)
          ENDDO
          IF(IDIV.EQ.1) THEN
            DO JGR=1,NGRP
              SUM=0.0D0
              DO IGR=1,NGRP
                SUM=SUM+RATE(JGR,IGR)*FLUX(IGR,ISOT)
              ENDDO
              RHS(JGR,JGR,ISOT)=-REAL(SUM)/FLUX(JGR,ISOT)
            ENDDO
          ELSE IF(IDIV.EQ.2) THEN
            DO IGR=1,NGRP
              SUM=0.0D0
              DO JGR=1,NGRP
                SUM=SUM+RATE(JGR,IGR)*AFLUX(JGR,ISOT)
              ENDDO
              RHS(IGR,IGR,ISOT)=-REAL(SUM)/AFLUX(IGR,ISOT)
            ENDDO
          ELSE IF(IDIV.EQ.3) THEN
            CALL SETARA(BASE,NGRP,IDLK)
            CALL SETARA(BASE,NGRP,IALK)
            DO JGR=1,NGRP
              SUM=0.0D0
              DO IGR=1,NGRP
                SUM=SUM+RATE(JGR,IGR)*FLUX(IGR,ISOT)
              ENDDO
              BASE(IDLK+JGR-1)=REAL(SUM)
            ENDDO
            DO IGR=1,NGRP
              SUM=0.0D0
              DO JGR=1,NGRP
                SUM=SUM+RATE(JGR,IGR)*AFLUX(JGR,ISOT)
              ENDDO
              BASE(IALK+IGR-1)=REAL(SUM)
            ENDDO
            CALL SETARA(BASE,NGRP,IV)
            CALL SETARA(BASE,NGRP,IW)
            CALL SETARA(BASE,2*(2*NGRP+1)*(2*NGRP),IDFF)
            CALL SETARA(BASE,2*(2*NGRP)*(2*NGRP+1),IDFF2)
            CALL DUO005(NGRP,BASE(IDLK),BASE(IALK),FLUX(1,ISOT),
     >      AFLUX(1,ISOT),BASE(IV),BASE(IW),BASE(IDFF),BASE(IDFF2))
            CALL RLSARA(BASE(IDFF2))
            CALL RLSARA(BASE(IDFF))
            DO IGR=1,NGRP
              DO JGR=1,NGRP
                RHS(IGR,JGR,ISOT)=RHS(IGR,JGR,ISOT)-BASE(IV+IGR-1)
     >          -BASE(IW+JGR-1)
              ENDDO
            ENDDO
            CALL RLSARA(BASE(IW))
            CALL RLSARA(BASE(IV))
            CALL RLSARA(BASE(IALK))
            CALL RLSARA(BASE(IDLK))
          ENDIF
        ELSE
          CALL LCMGET(IPLIB,HREAC,VECTOR)
          DO IGR=1,NGRP
            RHS(IGR,IGR,ISOT)=RHS(IGR,IGR,ISOT)+DENS(ISOT)*VECTOR(IGR)
          ENDDO
        ENDIF
        CALL LCMSIX(IPLIB,' ',2)
      ENDDO
      RETURN
      END
